#!/bin/bash

# This script installs all sources for a given bareboard target

# You can use ln for debugging (option --link).
CP="cp -p"

verbose=
objdir=install
debug=0
crossdir=cross-dir
gccdir=no-gcc-dir
libm=""
use_stdlayout=0

set -e

copy()
{
    if [ "$verbose" = "y" ]; then
      echo "Copying $1 to $2"
    fi
    $CP $1 $2
}

while [ $# -ne 0 ]; do
  case $1 in
      -v) verbose=y ;;
      -d) debug=`expr $debug + 1`;;
      --stdlayout) use_stdlayout=1 ;;
      --output=*) objdir=`echo $1 | sed -e "s/^--output=//"`;;
      --cross-dir=*) crossdir=`echo $1 | sed -e "s/^--cross-dir=//"`;;
      --objdir=*) objdir=`echo $1 | sed -e "s/^--objdir=//"`;;
      --libm=*) libm=`echo $1 | sed -e "s/^--libm=//"`;;
      --gcc-dir=*) gccdir=`echo $1 | sed -e "s/^--gcc-dir=//"`;;
      --link) CP="ln -s";;
      -*) echo "unknown option $1"; exit 1;;
      *) break
  esac
  shift
done

if [ $# -ne 2 ]; then
  echo "Usage: $0 [options] config gnat-src"
  echo "
 -v              verbose output
 -d              debug output
 --stdlayout     use adainclude/adalib directories
 --cross-dir=dir use dir for locating architecture specific files
 --objdir=dir    use dir for installing run time (default: install)
 --libm=file     use file as math library
 --link          use symbolic links instead of copying
 "
fi

config=$1
gnatsrc=$2

if [ -d $objdir ]; then
  echo "Object dir \"$objdir\" already exists"
  exit 1
fi

# Determine locations for common and architecture-specific files
common=common
arch=arch
lib=adalib

if [ "$use_stdlayout" = 1 ] ; then
  common=adainclude
  arch=adainclude
  lib=adalib
fi

# Create directories.
mkdir $objdir
mkdir -p $objdir/$arch
mkdir -p $objdir/$common
mkdir $objdir/obj
mkdir -p $objdir/$lib

discarded_sources="none"

# Sources for flexible run-times:
# mem_src: memcpy, memset, memcmp, memmove, last_chance_handler
mem_src="memory_set.ads memory_set.adb
	 memory_copy.ads memory_copy.adb
         memory_move.ads memory_move.adb
	 memory_compare.ads memory_compare.adb"
lch_src="last_chance_handler.ads last_chance_handler.adb"
# Last chance handler with tracebacks
lch_tb_src="a-exctra.ads
            s-traent.ads s-traent.adb
            s-tracon.ads s-tracon.adb
            a-elchha.ads a-elchha.adb
            g-debuti.ads g-debuti.adb"
lch_tb_pairs="a-elchha.ads:a-elchha-zfp.ads
              a-elchha.adb:a-elchha-zfp.adb
              a-exctra.ads:a-exctra-zfp.ads
              s-traceb.ads:s-traceb-cert.ads"
lch_tb_ppc_pairs="$lch_tb_pairs
                  s-traceb.adb:s-traceb-cert-ppc.adb
                  s-tracon.adb:s-tracon-ppc-eabi.adb"

img_src="s-imgint.ads s-imgint.adb
	 s-imglli.ads s-imglli.adb
	 s-imgboo.ads s-imgboo.adb
         s-imguns.ads s-imguns.adb
         s-imgllu.ads s-imgllu.adb"
#	 s-memory.ads s-memory.adb"
textio_src="a-textio.ads a-textio.adb text_io.ads"
textio_pairs="a-textio.ads:a-textio-zfp.ads a-textio.adb:a-textio-zfp.adb
              g-io-put.adb:g-io-put-stextio.adb"

extra_raven_files="$textio_src
                   s-bbcpsp.ads s-bbcpsp.adb
                   s-macres.ads s-macres.adb"

# Files that implement error and malloc.  For ravenscar full.
libc_files="s-c.ads
            s-cerrno.ads s-cerrno.adb
	    s-cmallo.ads s-cmallo.adb"
libc_pairs="s-c.ads:s-c-zfp.ads
            s-cerrno.ads:s-cerrno-zfp.ads
            s-cerrno.adb:s-cerrno-zfp.adb
	    s-cmallo.ads:s-cmallo-zfp.ads
            s-cmallo.adb:s-cmallo-zfp.adb
            s-cstrle.ads:s-cstrle-zfp.ads
            s-cstrle.adb:s-cstrle-zfp.adb"

# Files and pairs for ZCX (ravenscar-full only)
zcx_files="s-cstrle.ads s-cstrle.adb
           s-except.ads s-except.adb
           s-exctab.ads s-exctab.adb
           s-excmac.ads
           a-exexda.adb
           a-exexpr.adb
           a-exextr.adb
           a-exstat.adb
           a-excpol.adb
           raise-gcc.c raise.h
           adaint-xi.c"

zcx_pairs="s-stalib.ads:s-stalib.ads
           s-stalib.adb:s-stalib.adb
           a-except.adb:a-except-2005.adb
           a-except.ads:a-except-2005.ads
           a-exexpr.adb:a-exexpr-gcc.adb
           s-excmac.ads:s-excmac-gcc.ads"

zcx_copy()
{
 copy $PWD/src/tconfig.h $objdir/common
 copy $PWD/src/tsystem.h $objdir/common
 copy $gccdir/libgcc/unwind-pe.h $objdir/common
}

# libgcc in Ada (div, shifts)
libgcc_files="s-gcc.ads s-gcc.adb
              s-gccshi.ads s-gccshi.adb
              s-gccdiv.ads s-gccdiv.adb"

ppc_fpr_savres_files="powerpc/6xx/restfpr.S
                      powerpc/6xx/restxfpr.S
                      powerpc/6xx/savefpr.S"

ppc_gpr_savres_files="powerpc/6xx/restgpr.S
                      powerpc/6xx/restxgpr.S
                      powerpc/6xx/savegpr.S"

prep_arch_files="powerpc/prep/qemu-rom.ld
		 powerpc/prep/qemu-ram.ld
		 powerpc/prep/start-rom.S
		 powerpc/prep/start-ram.S
		 powerpc/prep/setup.S
		 powerpc/prep/s-ioport.ads powerpc/prep/s-ioport.adb"

ppc6xx_raven_files="powerpc/6xx/handler.S powerpc/6xx/context_switch.S"

prep_raven_files="$prep_arch_files $ppc6xx_raven_files"

w8349e_arch_files="powerpc/8349e/ram.ld
 		   powerpc/8349e/start-ram.S
		   powerpc/8349e/setup.S"

w8349e_raven_files="$w8349e_arch_files $ppc6xx_raven_files"

p2020_arch_files="powerpc/p2020/start-ram.S
		  powerpc/p2020/setup.S
		  powerpc/p2020/p2020.ld"

ppcspe_raven_files="powerpc/spe/handler.S powerpc/spe/context_switch.S"

p2020_raven_files="$p2020_arch_files $ppcspe_raven_files"

p55_arch_files="powerpc/p5566/start-bam.S
                powerpc/p5566/start-ram.S
                powerpc/p5566/start-flash.S
                powerpc/p5566/setup.S
                powerpc/p5566/setup-pll.S
		powerpc/p5566/bam.ld
		powerpc/p5566/flash.ld
		powerpc/p5566/ram.ld"

lmpmcm_arch_files="lmp/mcm/start.S"

tms570_arch_files="arm/tms570/tms570.ld
                   arm/tms570/sys_startup.S
                   arm/tms570/flash.ld
                   arm/tms570/monitor.ld
                   arm/tms570/hiram.ld
                   arm/tms570/loram.ld
                   arm/tms570/common.ld"
#                   arm/tms570/start-ram.S
#                   arm/tms570/start-rom.S"

raven_ppc_pairs="$textio_pairs
                 s-bbcppr.adb:s-bbcppr-ppc.adb
                 s-bbcppr.ads:s-bbcppr-ppc.ads
                 s-bbbosu.ads:s-bbbosu-ppc.ads
                 s-bbinte.adb:s-bbinte-ppc.adb
                 s-bbtime.adb:s-bbtime-ppc.adb"

lm3s_arch_files="arm/lm3s/lm3s-rom.ld
                 arm/lm3s/lm3s-ram.ld
                 arm/lm3s/start-rom.S
                 arm/lm3s/start-ram.S
                 arm/lm3s/setup_pll.adb"

stm32f4_arch_files="arm/stm32f4/stm32f4-rom.ld
                arm/stm32f4/stm32f4-ram.ld
                arm/stm32f4/start-rom.S
                arm/stm32f4/start-ram.S
                arm/stm32f4/setup_pll.adb"

sam4s_arch_files="arm/sam4s/sam4s-rom.ld
                  arm/sam4s/start-rom.S"

cortexm3_raven_files="arm/lm3s/context_switch.S arm/lm3s/handler.S"
cortexm4_raven_files="arm/stm32f4/handler.S"

extra_target_pairs=
extra_rts_files=

# The RTS which is based on. Used to copy file using Makefile.hie
case $config in
  zfp*/*) gnat_rts=zfp;;
  ravenscar-sfp/*) gnat_rts=ravenscar-sfp;;
  ravenscar-xtratum/*) gnat_rts=ravenscar-sfp;;
  ravenscar-minimal/*) gnat_rts=ravenscar-minimal;;
  ravenscar-full/*) gnat_rts=ravenscar;;
  ravenscar-full-xtratum/*) gnat_rts=ravenscar;;
  */*) echo "build-rts.sh: unknown runtime config $config"; exit 1;;
esac

if [ "$gnat_rts" = "ravenscar" ]; then
    img_src="" # No need to build img_src as they are already in the RTS.
fi

# Project file for zfp-support configs
zfp_support_prj_files="$PWD/support-prj/runtime.gpr
        $PWD/support-prj/runtime_build.gpr
        $PWD/support-prj/support.gpr"

# Sed command used to modify the .xml file to link with
# libgnat + libm + libgcc + libgnat
ADD_LIBM_SED_CMD='s@(\(.*\)/libgnat.a"@(\1/libgnat.a",\
       \1/libm.a",\
       "-lgcc",\
       \1/libgnat.a"@'

# Arch part of the config (ie the string after the /)
config_arch=$(echo $config | sed -e "s,.*/,,")

gnat_target=
gnat_cross=

case $config in
    "zfp-support/p55-prep")
        gnat_rts=
        # FIXME: use powerpc/prep/a-textio.adb ?
	arch_files="../powerpc-eabispe/qemu/start.s
	           ../powerpc-elf/qemu/c_io.c
		   ../powerpc-elf/qemu/ppc_qemu.ld"
        mem_src="$mem_src $lch_src
                 secondary_stack.ads secondary_stack.adb"
        for f in $zfp_support_prj_files; do copy $f $objdir/; done
	;;
    "zfp/p2020")
        gnat_target=powerpc-eabispe
        arch_files=$p2020_arch_files
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-bb.ads
                         s-bbbopa.ads
                         s-macres.ads s-macres.adb"
        extra_target_pairs="s-textio.adb:s-textio-p2020.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-bbbopa.ads:s-bbbopa-p2020.ads
                            s-macres.adb:s-macres-p2020.adb
                            $textio_pairs"
        copy $PWD/powerpc/p2020/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-sfp/p2020" )
        gnat_target=powerpc-eabispe
        arch_files=$p2020_raven_files
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$extra_raven_files s-bbbopa.ads"
        extra_target_pairs="s-textio.adb:s-textio-p2020.adb
                            $raven_ppc_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-spe.ads
                            s-bbcpsp.adb:s-bbcpsp-spe.adb
                            s-secsta.adb:s-secsta-ppc.adb
                            s-bbbopa.ads:s-bbbopa-p2020.ads
                            s-bbpara.ads:s-bbpara-ppc.ads
                            s-macres.adb:s-macres-p2020.adb
                            s-bbbosu.adb:s-bbbosu-p2020.adb
                            a-intnam.ads:a-intnam-xi-p2020.ads
                            system.ads:system-xi-e500v2-sfp.ads"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb"
        copy $PWD/powerpc/p2020/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-full/p2020" )
        gnat_target=powerpc-eabispe
	# For ZCX -lgcc is required (must be before libravenscar for strlen
	#  and other libc routines).
	# Math lib (-lm) depends on errnor (which is in libravenscar)
	# Need additional specs (to re-add crtbegin/crtend)
        sed -e \
         's@(\(.*\)/libgnat.a"@(\1/libgnat.a",\
         \1/libm.a",\
         "-lgcc",\
         \1/libgnat.a",\
         "--specs=${RUNTIME_DIR(ada)}/link.spec"@' \
            < powerpc/p2020/runtime.xml > $objdir/runtime.xml
	# Need to define CALL_init so that constructors (in particular
	#  ZCX tables registering) are called
        sed -e 's/ASMFLAGS := (/ASMFLAGS := ("-DCALL__init") \& (/' \
            < src/runtime_build.gpr > $objdir/runtime_build.gpr
	zcx_copy
        copy $PWD/powerpc/prep/link-zcx.spec $objdir/link.spec
        arch_files="$p2020_raven_files"
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$extra_raven_files $libc_files $zcx_files
                         s-bbbopa.ads"
        extra_target_pairs="s-textio.adb:s-textio-p2020.adb
                            $raven_ppc_pairs
                            $libc_pairs
                            $zcx_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-spe.ads
                            s-bbcpsp.adb:s-bbcpsp-spe.adb
                            s-bbbopa.ads:s-bbbopa-p2020.ads
                            s-bbpara.ads:s-bbpara-ppc.ads
                            s-macres.adb:s-macres-p2020.adb
                            s-bbbosu.adb:s-bbbosu-p2020.adb
                            a-intnam.ads:a-intnam-xi-p2020.ads
                            system.ads:system-xi-e500v2-full.ads"
        ;;
    "zfp/mpc5554")
        gnat_target=powerpc-eabispe
        arch_files="powerpc/mpc5554/5554.ld powerpc/mpc5554/start.S"
        mem_src="$mem_src
                 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb
                         $lch_tb_src"
        extra_target_pairs="s-textio.adb:s-textio-p55.adb
                            s-textio.ads:s-textio-zfp.ads
                            $textio_pairs
                            s-macres.adb:s-macres-p55.adb
                            $lch_tb_ppc_pairs"
        copy $PWD/powerpc/mpc5554/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "zfp/mpc5634")
        gnat_target=powerpc-eabispe
        arch_files="powerpc/mpc5634/5634.ld powerpc/mpc5634/start.S"
        mem_src="$mem_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb
                         $lch_tb_src"
        extra_target_pairs="s-textio.adb:s-textio-p55.adb
                            s-textio.ads:s-textio-zfp.ads
                            $textio_pairs
                            s-macres.adb:s-macres-p55.adb
                            $lch_tb_ppc_pairs"
        copy $PWD/powerpc/mpc5634/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "zfp/p5566")
        gnat_target=powerpc-eabispe
        arch_files=$p55_arch_files
        mem_src="$mem_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb
                         $lch_tb_src"
        extra_target_pairs="s-textio.adb:s-textio-p55.adb
                            s-textio.ads:s-textio-zfp.ads
                            $textio_pairs
                            s-macres.adb:s-macres-p55.adb
                            $lch_tb_ppc_pairs"
        copy $PWD/powerpc/p5566/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;

    "ravenscar-sfp/p5566" )
        gnat_target=powerpc-eabispe
        copy $PWD/powerpc/p5566/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        arch_files="$p55_arch_files $ppcspe_raven_files"
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$extra_raven_files s-bbbopa.ads"
        extra_target_pairs="s-textio.adb:s-textio-p55.adb
                            s-textio.ads:s-textio-zfp.ads
                            $raven_ppc_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-spe.ads
                            s-bbcpsp.adb:s-bbcpsp-spe.adb
                            s-secsta.adb:s-secsta-ppc.adb
                            s-macres.adb:s-macres-p55.adb
                            s-bbpara.ads:s-bbpara-p55.ads
                            s-bbbopa.ads:s-bbbopa-p55.ads
                            s-bbbosu.adb:s-bbbosu-p55.adb
                            a-intnam.ads:a-intnam-xi-p55.ads
                            system.ads:system-xi-e500v2-sfp.ads"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb"
        ;;
    "ravenscar-full/p5566" )
        gnat_target=powerpc-eabispe
	# For ZCX -lgcc is required (must be before libravenscar for strlen
	#  and other libc routines).
	# Math lib (-lm) depends on errnor (which is in libravenscar)
	# Need additional specs (to re-add crtbegin/crtend)
        sed -e \
         's@(\(.*\)/libgnat.a"@(\1/libgnat.a",\
         \1/libm.a",\
         "-lgcc",\
         \1/libgnat.a",\
         "--specs=${RUNTIME_DIR(ada)}/link.spec"@' \
         -e 's/("-nostdlib", "-lgcc")/()/' \
            < powerpc/p5566/runtime.xml > $objdir/runtime.xml
	# Need to define CALL_init so that constructors (in particular
	#  ZCX tables registering) are called
        sed -e 's/ASMFLAGS := (/ASMFLAGS := ("-DCALL__init") \& (/' \
            < src/runtime_build.gpr > $objdir/runtime_build.gpr
	zcx_copy
        copy $PWD/powerpc/prep/link-zcx.spec $objdir/link.spec
        arch_files="$p55_arch_files $ppcspe_raven_files"
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$extra_raven_files $libc_files $zcx_files
                         s-bbbopa.ads"
        extra_target_pairs="s-textio.adb:s-textio-p55.adb
                            $raven_ppc_pairs
                            $libc_pairs
                            $zcx_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-spe.ads
                            s-bbcpsp.adb:s-bbcpsp-spe.adb
                            s-macres.adb:s-macres-p55.adb
                            s-bbpara.ads:s-bbpara-p55.ads
                            s-bbbopa.ads:s-bbbopa-p55.ads
                            s-bbbosu.adb:s-bbbosu-p55.adb
                            a-intnam.ads:a-intnam-xi-p55.ads
                            system.ads:system-xi-e500v2-full.ads"
        ;;
    "zfp/mcm")
        gnat_target=lmp-elf
        arch_files=$lmpmcm_arch_files
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb"
        extra_target_pairs="s-textio.adb:s-textio-mcm.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-macres.adb:s-macres-mcm.adb
                            system.ads:system-xi-lmp.ads
                            $textio_pairs"
        copy $PWD/lmp/mcm/runtime.xml $objdir/runtime.xml
        copy $PWD/lmp/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "zfp/prep")
        gnat_target=powerpc-elf
        arch_files=$prep_arch_files
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb"
        extra_target_pairs="s-textio.adb:s-textio-prep.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-macres.adb:s-macres-prep.adb
                            $textio_pairs"
        copy $PWD/powerpc/prep/runtime.xml $objdir/runtime.xml
        copy $PWD/powerpc/prep/zfp.gpr $objdir/runtime.gpr
        copy $PWD/powerpc/prep/zfp_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-sfp/prep" \
  | "ravenscar-sfp/prep-nofpu" )
        gnat_target=powerpc-elf
        if [ "$config" = "ravenscar-sfp/prep" ]; then
          sed -e "s/zfp/sfp/g" \
              < powerpc/prep/runtime.xml > $objdir/runtime.xml
          sed -e "s/zfp/sfp/g" \
              < powerpc/prep/zfp.gpr > $objdir/runtime.gpr
          sed -e "s/zfp/sfp/g" \
              < powerpc/prep/zfp_build.gpr > $objdir/runtime_build.gpr
        else
          sed -e "s/zfp/sfp/g" -e "s/-mhard-float/-msoft-float/g" \
              < powerpc/prep/runtime.xml > $objdir/runtime.xml
          sed -e "s/zfp/sfp/g" -e "s/-mhard-float/-msoft-float/g" \
            < powerpc/prep/zfp.gpr > $objdir/runtime.gpr
          copy $PWD/powerpc/prep/zfp_build.gpr $objdir/runtime_build.gpr
        fi
        arch_files=$prep_raven_files
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$extra_raven_files"
        extra_target_pairs="s-textio.adb:s-textio-prep.adb
                            $raven_ppc_pairs
	                    s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-6xx.ads
                            s-bbcpsp.adb:s-bbcpsp-6xx.adb
                            s-bbpara.ads:s-bbpara-ppc.ads
                            s-secsta.adb:s-secsta-ppc.adb
                            s-macres.adb:s-macres-prep.adb
                            a-intnam.ads:a-intnam-xi-ppc.ads
                            s-bbbosu.adb:s-bbbosu-prep.adb
                            system.ads:system-xi-ppc-sfp.ads"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb"
        ;;
    "ravenscar-full/prep" )
        gnat_target=powerpc-elf
        # Ravenscar depends on -lm (for math routines), which also
        # depends on ravenscar (for errno).
        sed -e 's@(\(.*\)/libzfp.a"@(\1/libravenscar.a",\
                \1/libm.a", "-lgcc", \1/libravenscar.a"@' \
            < powerpc/prep/zfp.gpr > $objdir/runtime.gpr
	# For ZCX -lgcc is required (must be before libravenscar for strlen
	#  and other libc routines).
	# Math lib (-lm) depends on errnor (which is in libravenscar)
	# Need additional specs (to re-add crtbegin/crtend)
        sed -e \
         's@"\$.*/libzfp.a"@"${RUNTIME_DIR(ada)}/adalib/libravenscar.a",\
         "${RUNTIME_DIR(ada)}/adalib/libm.a",\
         "-lgcc",\
         "${RUNTIME_DIR(ada)}/adalib/libravenscar.a",\
         "--specs=${RUNTIME_DIR(ada)}/link.spec"@' \
            < powerpc/prep/runtime.xml > $objdir/runtime.xml
	# Need to define CALL_init so that constructors (in particular
	#  ZCX tables registering) are called
        sed -e "/Library_Name/s/gnat/ravenscar/g" \
            -e 's/ASMFLAGS := (/ASMFLAGS := ("-DCALL__init") \& (/' \
            < src/runtime_build.gpr > $objdir/runtime_build.gpr
        arch_files="$prep_raven_files"
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$extra_raven_files $libc_files $zcx_files"
        extra_target_pairs="s-textio.adb:s-textio-prep.adb
                            $raven_ppc_pairs
                            $libc_pairs
                            $zcx_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-6xx.ads
                            s-bbcpsp.adb:s-bbcpsp-6xx.adb
                            s-bbpara.ads:s-bbpara-ppc.ads
                            s-macres.adb:s-macres-prep.adb
                            a-intnam.ads:a-intnam-xi-ppc.ads
                            s-bbbosu.adb:s-bbbosu-prep.adb
                            system.ads:system-xi-ppc-full.ads"
        copy $PWD/powerpc/prep/link-zcx.spec $objdir/link.spec
	zcx_copy
        ;;
    "zfp/8349e")
        gnat_target=powerpc-elf
        arch_files=$w8349e_arch_files
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-bb.ads
                         s-bbbopa.ads
                         s-macres.ads s-macres.adb"
        extra_target_pairs="s-textio.adb:s-textio-p2020.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-bbbopa.ads:s-bbbopa-8349e.ads
                            $textio_pairs
                            s-macres.adb:s-macres-8349e.adb"
        copy $PWD/powerpc/8349e/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-sfp/8349e")
        gnat_target=powerpc-elf
        copy $PWD/powerpc/8349e/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        arch_files=$w8349e_raven_files
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$extra_raven_files
                         s-bbbopa.ads
                         $libgcc_files
                         g-intpri.ads"
        extra_target_pairs="s-textio.adb:s-textio-p2020.adb
                            $raven_ppc_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-6xx.ads
                            s-bbcpsp.adb:s-bbcpsp-6xx.adb
                            s-secsta.adb:s-secsta-ppc.adb
                            s-bbbopa.ads:s-bbbopa-${config_arch}.ads
                            s-bbbosu.adb:s-bbbosu-8349e.adb
                            s-bbpara.ads:s-bbpara-ppc.ads
                            a-intnam.ads:a-intnam-xi-8349e.ads
                            system.ads:system-xi-ppc-sfp.ads
                            s-macres.adb:s-macres-8349e.adb"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb"
        ;;
    "ravenscar-minimal/8349e" \
     | "ravenscar-minimal/8321e")
        gnat_target=powerpc-elf
        copy $PWD/powerpc/8349e/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        copy $PWD/powerpc/8349e/notes.txt $objdir/notes.txt
        arch_files="$w8349e_raven_files $ppc_gpr_savres_files"
        discarded_sources="s-sssita.ads s-sssita.adb"
        extra_rts_files="$textio_src
                         s-bbcpsp.ads s-bbcpsp.adb
                         s-bbbopa.ads
                         s-tracon.ads s-tracon.adb
                         g-secsta.ads
                         $libgcc_files
                         g-intpri.ads"
        extra_target_pairs="s-textio.adb:s-textio-p2020.adb
                            $raven_ppc_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            s-bbcpsp.ads:s-bbcpsp-6xx.ads
                            s-bbcpsp.adb:s-bbcpsp-6xx.adb
                            s-secsta.adb:s-secsta-ppc.adb
                            s-bbbopa.ads:s-bbbopa-${config_arch}.ads
                            s-bbbosu.adb:s-bbbosu-8349e.adb
                            s-bbpara.ads:s-bbpara-ppc.ads
                            a-intnam.ads:a-intnam-xi-${config_arch}.ads
                            system.ads:system-xi-ppc-minimal.ads
                            s-macres.adb:s-macres-8349e.adb
                            $lch_tb_ppc_pairs
                            s-secsta.ads:s-secsta-zfp.ads
                            s-memory.adb:s-memory-raven-min.adb
                            g-secsta.ads:g-secsta-minimal.ads"
	img_src="" # Already part of minimal profile
        ;;
    "zfp/tms570" | "zfp/tms570-sci")
        gnat_target="arm-eabi"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb"
        extra_target_pairs="system.ads:system-xi-arm.ads
                            s-textio.ads:s-textio-zfp.ads
                            $textio_pairs"
        if [ "$config" = "zfp/tms570-sci" ]; then
            arch_files="$tms570_arch_files
                        arm/tms570/start-ram.S
                        arm/tms570/start-rom.S"
            extra_rts_files="$extra_rts_files s-macres.ads s-macres.adb"
            extra_target_pairs="$extra_target_pairs
                                s-macres.adb:s-macres-tms570.adb
                                s-textio.adb:s-textio-tms570-sci.adb"
            sed -e '/LOADER/s/PROBE/LORAM/' \
		<  $PWD/arm/tms570/runtime.xml > $objdir/runtime.xml
	else
            arch_files="$tms570_arch_files arm/tms570/crt0.S"
            extra_target_pairs="$extra_target_pairs
                                s-textio.adb:s-textio-tms570.adb"
            copy $PWD/arm/tms570/runtime.xml $objdir/runtime.xml
	fi
        copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-sfp/tms570")
        gnat_target="arm-eabi"
        discarded_sources="s-sssita.ads s-sssita.adb"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb"
        arch_files="$tms570_arch_files arm/tms570/crt0.S"
	extra_rts_files="$textio_src"
        extra_target_pairs="a-intnam.ads:a-intnam-xi-tms570.ads
                            system.ads:system-xi-arm-sfp.ads
                            s-textio.adb:s-textio-tms570.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-bbcppr.adb:s-bbcppr-arm.adb
                            s-bbpara.ads:s-bbpara-tms570.ads
                            s-bbbosu.adb:s-bbbosu-tms570.adb
                            s-parame.ads:s-parame-xi-small.ads
                            $textio_pairs"
        copy $PWD/arm/$config_arch/runtime.xml $objdir/runtime.xml
        copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-full/tms570" | "ravenscar-full/tms570-sci")
        gnat_target="arm-eabi"
        discarded_sources="s-sssita.ads s-sssita.adb"
        arch_files="$tms570_arch_files arm/tms570/crt0.S"
	extra_rts_files="$textio_src $libc_files $zcx_files"
        extra_target_pairs="a-intnam.ads:a-intnam-xi-tms570.ads
                            system.ads:system-xi-arm-full.ads
                            s-textio.ads:s-textio-zfp.ads
                            s-bbcppr.adb:s-bbcppr-arm.adb
                            s-bbpara.ads:s-bbpara-tms570.ads
                            s-bbbosu.adb:s-bbbosu-tms570.adb
                            s-traceb.adb:s-traceb-xi-arm.adb
                            s-parame.ads:s-parame-xi-small.ads
                            $zcx_pairs
                            $textio_pairs
                            $libc_pairs"
        if [ "$config" = "ravenscar-full/tms570-sci" ]; then
            arch_files="$tms570_arch_files
                        arm/tms570/start-ram.S
                        arm/tms570/start-rom.S"
            extra_rts_files="$extra_rts_files s-macres.ads s-macres.adb"
            extra_target_pairs="$extra_target_pairs
                                s-macres.adb:s-macres-tms570.adb
                                s-textio.adb:s-textio-tms570-sci.adb"
            sed -e '/LOADER/s/PROBE/LORAM/' -e "$ADD_LIBM_SED_CMD" \
	       -e '/Asm_Required_Switches := /s/()/("-D__ZCX__")/' \
		<  $PWD/arm/tms570/runtime.xml > $objdir/runtime.xml
	else
            arch_files="$tms570_arch_files arm/tms570/crt0.S"
            extra_target_pairs="$extra_target_pairs
                                s-textio.adb:s-textio-tms570.adb"
           sed -e "$ADD_LIBM_SED_CMD" \
	       -e '/Asm_Required_Switches := /s/()/("-D__ZCX__")/' \
            < arm/tms570/runtime.xml > $objdir/runtime.xml
	fi
	copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
	copy $PWD/src/tconfig.h $objdir/adainclude
	copy $PWD/src/tsystem.h $objdir/adainclude
	copy $gccdir/libgcc/unwind-pe.h $objdir/adainclude
	;;
    "ravenscar-xtratum/tms570" | "ravenscar-full-xtratum/tms570")
        gnat_target=arm-eabi
        discarded_sources="s-sssita.ads s-sssita.adb"
        if [ "$config" = "ravenscar-xtratum/tms570" ]; then
          mem_src="$mem_src
                   $lch_src
		   s-memory.ads s-memory.adb"
        fi
        arch_files="arm/xtratum/cswitch.S
                    arm/xtratum/init.S
                    arm/xtratum/xtratum.ld"
	extra_rts_files="$textio_src
                         s-macres.ads s-macres.adb"
        extra_target_pairs="a-intnam.ads:a-intnam-xi-tms570.ads
                            s-textio.adb:s-textio-xtratum.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-bbcppr.adb:s-bbcppr-arm-xtratum.adb
                            s-bbpara.ads:s-bbpara-xtratum-tms570.ads
                            s-bbbosu.adb:s-bbbosu-xtratum-arm.adb
                            s-parame.ads:s-parame-xi-small.ads
                            s-macres.adb:s-macres-xtratum.adb
                            s-bbexti.adb:s-bbexti-xtratum.adb
                            $textio_pairs"
        if [ "$config" = "ravenscar-xtratum/tms570" ]; then
            extra_target_pairs="$extra_target_pairs
                                system.ads:system-xi-arm-sfp.ads"
            copy $PWD/arm/xtratum/runtime.xml $objdir/runtime.xml
	else
	    extra_rts_files="$extra_rts_files $libc_files"
            extra_target_pairs="$extra_target_pairs
                                $libc_pairs
                                s-traceb.adb:s-traceb-xi-arm.adb
                                system.ads:system-xi-arm-full.ads"
            sed -e "$ADD_LIBM_SED_CMD" \
		< arm/xtratum/runtime.xml > $objdir/runtime.xml
	fi
        copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "zfp/stm32f4")
        gnat_target=arm-eabi
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        arch_files="$stm32f4_arch_files"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb s-stm32f.ads"
        extra_target_pairs="s-textio.adb:s-textio-stm32f4.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-macres.adb:s-macres-cortexm3.adb
                            $textio_pairs
                            system.ads:system-xi-arm.ads"
        copy $PWD/arm/$config_arch/runtime.xml $objdir/runtime.xml
        copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-sfp/stm32f4")
        gnat_target=arm-eabi
        discarded_sources="s-sssita.ads s-sssita.adb"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb"
        arch_files="$stm32f4_arch_files $cortexm4_raven_files"
        extra_rts_files="$textio_src s-macres.adb s-macres.ads s-stm32f.ads"
        extra_target_pairs="a-intnam.ads:a-intnam-xi-stm32f4.ads
                            system.ads:system-xi-cortexm4-sfp.ads
                            s-macres.adb:s-macres-cortexm3.adb
                            s-textio.adb:s-textio-stm32f4.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-bbcppr.adb:s-bbcppr-armv7m.adb
                            s-bbpara.ads:s-bbpara-stm32f4.ads
                            s-bbbosu.adb:s-bbbosu-stm32f4.adb
                            s-parame.ads:s-parame-xi-small.ads
                            $textio_pairs"
        copy $PWD/arm/$config_arch/runtime.xml $objdir/runtime.xml
        copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-full/stm32f4")
        gnat_target=arm-eabi
        discarded_sources="s-sssita.ads s-sssita.adb"
        arch_files="$stm32f4_arch_files $cortexm4_raven_files"
        extra_rts_files="$textio_src $libc_files
                         s-stm32f.ads s-macres.ads s-macres.adb"
        extra_target_pairs="a-intnam.ads:a-intnam-xi-stm32f4.ads
                            system.ads:system-xi-arm-full.ads
                            s-macres.adb:s-macres-cortexm3.adb
                            s-textio.adb:s-textio-stm32f4.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-bbcppr.adb:s-bbcppr-armv7m.adb
                            s-bbpara.ads:s-bbpara-stm32f4.ads
                            s-bbbosu.adb:s-bbbosu-stm32f4.adb
                            s-parame.ads:s-parame-xi-small.ads
                            s-traceb.adb:s-traceb-xi-arm.adb
                            $libc_pairs
                            $textio_pairs"
        copy $PWD/arm/$config_arch/runtime.xml $objdir/runtime.xml
        copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "zfp/lm3s")
        gnat_target=arm-eabi
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        arch_files="$lm3s_arch_files"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb"
        extra_target_pairs="s-textio.adb:s-textio-lm3s.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-macres.adb:s-macres-cortexm3.adb
                            $textio_pairs
                            system.ads:system-xi-arm.ads"
        copy $PWD/arm/$config_arch/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-sfp/lm3s")
        gnat_target=arm-eabi
        discarded_sources="s-sssita.ads s-sssita.adb"
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb"
        arch_files="$lm3s_arch_files $cortexm3_raven_files"
        extra_rts_files="$extra_raven_files"
        extra_target_pairs="s-textio.adb:s-textio-lm3s.adb
                            s-bbcppr.adb:s-bbcppr-cortexm3.adb
                            s-bbcppr.ads:s-bbcppr-ppc.ads
                            s-bbcpsp.ads:s-bbcpsp-cortexm3.ads
                            s-bbcpsp.adb:s-bbcpsp-cortexm3.adb
                            s-bbpara.ads:s-bbpara-cortexm3.ads
                            s-bbbosu.ads:s-bbbosu-ppc.ads
                            s-bbbosu.adb:s-bbbosu-cortexm3.adb
                            s-bbinte.adb:s-bbinte-ppc.adb
                            s-bbtime.adb:s-bbtime-tick.adb
                            s-macres.adb:s-macres-cortexm3.adb
                            s-parame.ads:s-parame-xi-small.ads
                            $textio_pairs
                            s-multip.adb:s-multip-raven-default.adb
                            system.ads:system-xi-cortexm3-sfp.ads"
        copy $PWD/arm/$config_arch/runtime.xml $objdir/runtime.xml
        copy $PWD/src/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "zfp/sam4s")
        gnat_target=arm-eabi
        mem_src="$mem_src $lch_src
		 s-memory.ads s-memory.adb
                 secondary_stack.ads secondary_stack.adb"
        arch_files="$sam4s_arch_files"
        extra_rts_files="$textio_src s-textio.ads s-textio.adb
                         s-macres.ads s-macres.adb"
        extra_target_pairs="s-textio.adb:s-textio-null.adb
                            s-textio.ads:s-textio-zfp.ads
                            s-macres.adb:s-macres-cortexm3.adb
                            $textio_pairs
                            system.ads:system-xi-arm.ads"
        copy $PWD/arm/$config_arch/runtime.xml $objdir/runtime.xml
        copy $PWD/arm/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "zfp/erc32" \
  | "zfp/leon"  \
  | "zfp/leon3" \
  | "ravenscar-sfp/erc32"  \
  | "ravenscar-sfp/leon"   \
  | "ravenscar-sfp/leon3"  \
  | "ravenscar-full/erc32" \
  | "ravenscar-full/leon"  \
  | "ravenscar-full/leon3")
        gnat_target="${config_arch}-elf"
        mem_src="$mem_src $lch_src"
        arch_files=$(sed "s,^,$crossdir/," sparc/$config_arch/arch_files )

        copy $PWD/sparc/$config_arch/runtime.gpr $objdir/runtime.gpr
        copy $PWD/sparc/runtime_build.gpr $objdir/runtime_build.gpr
        ;;
    "ravenscar-xtratum/leon3" | "ravenscar-full-xtratum/leon3")
        # Using target ERC32 because the dependencies on the concrete target
        # should be handled by XtratuM. Two important reasons to choose ERC32
        # are 1) I would get multiprocessor support when using LEON3, and
        # 2) I would get undesired support for cache control and atomic
        # operations though alternate address spaces (see s-musplo-leon.adb).
        # We need to avoid this because 1) on XtratuM multiprocessors would be
        # handled by the hypervisor, and 2) cache control and alternate address
        # spaces are privileged operations not available in user mode.
        gnat_target=erc32-elf
	gnat_cross=leon3-elf
        discarded_sources="s-bbsuer.ads
                           s-sssita.ads
                           s-sssita.adb"
        extra_target_pairs="s-textio.adb:s-textio-xtratum.adb
                            s-bbpara.ads:s-bbpara-xtratum-sparc.ads
                            s-bcpith.adb:s-bcpith-xtratum-sparc.adb
	                    s-multip.adb:s-multip-raven-default.adb
                            a-intnam.ads:a-intnam-xi-leon3.ads
                            s-bbbosu.adb:s-bbbosu-xtratum.adb
                            g-io-put.adb:g-io-put-xtratum.adb
                            s-bbexti.adb:s-bbexti-xtratum.adb"
        copy $PWD/sparc/xtratum/runtime.xml $objdir/runtime.xml
        copy $PWD/sparc/xtratum/runtime.gpr $objdir/runtime.gpr
        copy $PWD/sparc/runtime_build.gpr $objdir/runtime_build.gpr
        # libxal from XtratuM provides already memcpy, memcmp and memset
        mem_src="memory_move.ads memory_move.adb"
        if [ "$config" = "ravenscar-xtratum/leon3" ]; then
            # We want this run time to have the same functionality as
            # the Ravenscar SFP one.
            img_src=""
        fi
        ;;
    *)
        echo "error: unknown config \"$config\""
        exit 1
        ;;
esac

# Build list of sources.
if [ "$gnat_rts" != "" ]; then
  # Needed if the script is invoked from a Makefile
  unset MAKEFLAGS MAKEOVERRIDES MAKELEVEL
  defs=`make -f $gnatsrc/Makefile.hie RTS=$gnat_rts \
          TARGET=$gnat_target GNAT_SRC_DIR=$gnatsrc show-sources`

  # Get them.
  eval $defs

  if [ $debug -ge 1 ]; then
    echo make -f $gnatsrc/Makefile.hie RTS=$gnat_rts \
          TARGET=$gnat_target GNAT_SRC_DIR=$gnatsrc show-sources
    echo "Defs:"
    echo $defs
    echo "LIBGNAT_SOURCES=$LIBGNAT_SOURCES"
    echo "LIBGNARL_SOURCES=$LIBGNARL_SOURCES"
    echo "LIBGNAT_NON_COMPILABLE_SOURCES=$LIBGNAT_NON_COMPILABLE_SOURCES"
  fi

  set -e

  # Remove pairs from extra_target_pairs of TARGET_PAIRS.
  # This allow to remove an override in TARGET_PAIRS.
  rmsedcmd="-e s:XYZ:XYZ:"
  for i in `echo $extra_target_pairs | sed -e 's/:[a-z0-9-]*\.ad.//g'`; do
    rmsedcmd="$rmsedcmd -e s/$i:[a-z0-9-]*\.ad.//"
  done
  # Avoid to have an empty command
  sedcmd="-e s:XYZ:XYZ:"
  for i in $extra_target_pairs `echo $TARGET_PAIRS | sed $rmsedcmd`; do
    sedcmd="$sedcmd -e s:$i:"
  done

  # Create the list of source files.
  all_sources="$LIBGNAT_SOURCES $LIBGNARL_SOURCES $LIBGNAT_NON_COMPILABLE_SOURCES"

  for f in $discarded_sources; do
    all_sources=`echo $all_sources | sed -e "s/$f//"`
  done

  if [ "$verbose" = "y" ]; then
    echo "Sources:"
    echo $all_sources
  fi

  # Copy sources.
  for f in $all_sources; do
    dest=$objdir/$common/$f

    pairf=`echo $f | sed $sedcmd`

    case $f in
       system.ads | s-bbbosu.adb | a-intnam.ads | s-textio.adb \
            | s-macres.adb | s-bbbopa.ads | *.ld)
         dest=$objdir/$arch/$f ;;
    esac

    copy $gnatsrc/$pairf $dest
  done
fi

# Copy some zfp sources
for f in $mem_src; do
   copy $PWD/../zfp-src/$f $objdir/$common/$f
done

for f in $img_src $extra_rts_files; do
   copy $gnatsrc/`echo $f | sed $sedcmd` $objdir/$common/$f
done

# Copy arch files
for f in $arch_files; do
   case $f in
       /*) copy $f $objdir/$arch/`basename $f`;;
     *.ld | *.specs)
           if [ "$use_stdlayout" = 1 ] ; then
             copy $PWD/$f $objdir/$lib
           else
             copy $PWD/$f $objdir/$arch
           fi ;;
        *) copy $PWD/$f $objdir/$arch/`basename $f`;;
   esac
done

# Create dummy libgnarl for compatibility with standard gnatmake.
# This is a kludge, we really should change gnatbind ???
if [ "$use_stdlayout" = 1 ] ; then
    ${gnat_cross:-$gnat_target}-ar r $objdir/$lib/libgnarl.a
fi

# Create rts path files
if [ "$gnat_rts" != "" ]; then
    echo $lib > $objdir/ada_object_path
    (echo $common;echo $arch)|uniq > $objdir/ada_source_path
fi

# Copy libm.
if [ "$libm" != "" ]; then
  copy $libm $objdir/$lib
fi

exit 0
